@@ -216,6 +216,7 @@ extern Pixel		FPselectcolor;
 
 
 /* number of rectangles to allocate */
+#define NUM_TITLE_GADGET_RECTS 4
 #define NUM_MATTE_TS_RECTS	(6)
 #define NUM_MATTE_BS_RECTS	(6)
 
@@ -241,7 +242,6 @@ extern Pixel		FPselectcolor;
 #define NUM_STATIC_TOP_RECTS	(2*ICON_INTERNAL_SHADOW_WIDTH)
 #define NUM_STATIC_BOTTOM_RECTS	(2*ICON_INTERNAL_SHADOW_WIDTH)
 
-
 /* client frame shadow widths */
 #ifdef WSM
 #define FRAME_EXTERNAL_SHADOW_WIDTH	(wmGD.frameExternalShadowWidth)
@@ -804,6 +804,7 @@ typedef struct _MenuSpec
     Context	  currentContext;
     Widget	  menuWidget;      /* RowColumn widget */
     unsigned int  whichButton;    /* tracks whichButton resource for top menu */
+	unsigned int  width;
     unsigned int  height;          /* height of top menu */
     MenuItem	 *menuItems;       /* linked list of MenuItem structures */
     MenuButton   *menuButtons;     /* array of MenuButton structures */
